@@ -1,4 +1,5 @@
 import copy
+from collections import namedtuple
 
 from django.db.models import Prefetch
 from django.db.models.fields.related import ManyToManyRel, ManyToOneRel
@@ -48,6 +49,13 @@ class RequestQueryParserMixin(object):
         request.parsed_restql_query = parsed_restql_query
         return parsed_restql_query
 
+    @classmethod
+    def get_nested_field_from_dict(cls, nested_field):
+        field_name = [name for name in nested_field][0]
+        field = {"name": field_name, "fields": nested_field[field_name]}
+        NestedField = namedtuple("NestedField", field.keys())
+        return NestedField(**field)
+
 
 class QueryArgumentsMixin(RequestQueryParserMixin):
     """Mixin for converting query arguments into query parameters"""
@@ -80,14 +88,15 @@ class QueryArgumentsMixin(RequestQueryParserMixin):
                 query_params.update({
                     name: value
                 })
+
         for field in parsed_query['include']:
             if isinstance(field, dict):
-                for sub_field, sub_parsed_query in field.items():
-                    nested_query_params = self.build_query_params(
-                        sub_parsed_query,
-                        parent=prefix + sub_field
-                    )
-                    query_params.update(nested_query_params)
+                nested_field = self.get_nested_field_from_dict(field)
+                nested_query_params = self.build_query_params(
+                    nested_field.fields,
+                    parent=prefix + nested_field.name
+                )
+                query_params.update(nested_query_params)
         return query_params
 
     def inject_query_params_in_req(self, request):
@@ -133,7 +142,7 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
         assert not(is_query_kwarg_set and is_parsed_query_kwarg_set), msg
 
         # flag to toggle using restql fields
-        self._use_restql_fields = False
+        self._ready_to_use_dynamic_fields = False
 
         # initialize parsed restql query
         self.parsed_restql_query = None
@@ -143,7 +152,7 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
 
     def to_representation(self, instance):
         # Activate using restql fields
-        self._use_restql_fields = True
+        self._ready_to_use_dynamic_fields = True
 
         if self.restql_kwargs["return_pk"]:
             return instance.pk
@@ -200,14 +209,7 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
             return False
 
     @staticmethod
-    def is_valid_alias(alias, field):
-        if alias == field:
-            msg = (
-                "Invalid definition of `%s` alias on `%s` field, "
-                "Alias can not be the same as the field name."
-            ) % (alias, field)
-            raise ValidationError(msg, code="invalid")
-
+    def is_valid_alias(alias):
         if len(alias) > restql_settings.MAX_ALIAS_LEN:
             msg = (
                 "The length of `%s` alias has exceeded "
@@ -223,7 +225,7 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
                 all_fields.keys(),
                 raise_exception=True
             )
-            self.is_valid_alias(alias, field)
+            self.is_valid_alias(alias)
             all_fields[alias] = all_fields[field]
         return all_fields
 
@@ -262,27 +264,44 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
                 continue
             if isinstance(field, dict):
                 # Nested field
-                for nested_field in field:
-                    self.is_field_found(
-                        nested_field,
-                        all_field_names,
-                        raise_exception=True
-                    )
-                    self.is_nested_field(
-                        nested_field,
-                        all_fields[nested_field],
-                        raise_exception=True
-                    )
-                allowed_nested_fields.update(field)
+                nested_field = self.get_nested_field_from_dict(field)
+
+                alias = self.parsed_restql_query["aliases"].get(
+                    nested_field.name,
+                    nested_field.name
+                )
+
+                self.is_field_found(
+                    nested_field.name,
+                    all_field_names,
+                    raise_exception=True
+                )
+                self.is_nested_field(
+                    nested_field.name,
+                    all_fields[nested_field.name],
+                    raise_exception=True
+                )
+                allowed_nested_fields.update({alias: nested_field.fields})
             else:
                 # Flat field
+                alias = self.parsed_restql_query["aliases"].get(field, field)
                 self.is_field_found(field, all_field_names, raise_exception=True)
-                allowed_flat_fields.append(field)
+                allowed_flat_fields.append(alias)
 
         # Keep track of nested fields for future reference from child
         # serializers
         self.nested_fields = allowed_nested_fields
 
+        def get_duplicates(items):
+            unique = []
+            repeated = []
+            for item in items:
+                if item not in unique:
+                    unique.append(item)
+                else:
+                    repeated.append(item)
+            return repeated
+
         included_and_excluded_fields = (
             allowed_flat_fields +
             list(allowed_nested_fields.keys()) +
@@ -294,35 +313,17 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
             len(set(included_and_excluded_fields))
 
         if including_or_excluding_field_more_than_once:
-            unique_fields = []
-            repeated_fields = []
-            for field in included_and_excluded_fields:
-                if field not in unique_fields:
-                    unique_fields.append(field)
-                else:
-                    repeated_fields.append("`%s`" % field)
+            repeated_fields = get_duplicates(included_and_excluded_fields)
             msg = (
-                "QueryFormatError on %s field(s), "
-                "Can not include/exclude a field more than once"
-            ) % ", ".join(repeated_fields)
+                "QueryFormatError: You have either "
+                "included/excluded a field more than once, "  # e.g {id, id}
+                "used the same alias more than once, "  # e.g {x: name, x: age}
+                "used a field name as an alias to another field or "  # e.g {id, id: age} Here age's not a parent
+                "renamed a field and included/excluded it again, "  # e.g {ID: id, id}
+                "The list of fields which led to this error is %s."
+            ) % str(repeated_fields)
             raise ValidationError(msg, "invalid")
 
-        def intersection(lst1, lst2):
-            temp = set(lst2)
-            return [value for value in lst1 if value in temp]
-
-        repeated_aliased_fields = intersection(
-            included_and_excluded_fields,
-            self.parsed_restql_query["aliases"]
-        )
-
-        if repeated_aliased_fields:
-            msg = (
-                "QueryFormatError on %s field(s), "
-                "Can not include/exclude aliased field more than once"
-            ) % ", ".join(repeated_aliased_fields)
-            raise ValidationError(msg, code="invalid")
-
         if excluded_fields:
             # Here we are sure that self.parsed_restql_query["exclude"]
             # is not empty which means the user specified fields to exclude,
@@ -360,16 +361,6 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
 
     @cached_property
     def restql_fields(self):
-        request = self.context.get('request')
-
-        is_not_a_request_to_process = (
-            self.restql_kwargs["disable_dynamic_fields"] or
-            not (request is None or self.has_restql_query_param(request))
-        )
-
-        if is_not_a_request_to_process:
-            return self.allowed_fields
-
         is_top_retrieve_request = (
             self.field_name is None and
             self.parent is None
@@ -419,20 +410,24 @@ class DynamicFieldsMixin(RequestQueryParserMixin):
         return parser.parse(self.restql_kwargs["query"])
 
     def get_parsed_restql_query(self):
+        request = self.context.get('request')
+
         if self.restql_kwargs["query"] is not None:
             # Get from query kwarg
             return self.get_parsed_restql_query_from_query_kwarg()
         elif self.restql_kwargs["parsed_query"] is not None:
             # Get from parsed_query kwarg
             return self.restql_kwargs["parsed_query"]
-        # Get from request query parameter
-        request = self.context.get('request', None)
-        return self.get_parsed_restql_query_from_req(request)
+        elif request is not None and self.has_restql_query_param(request):
+            # Get from request query parameter
+            return self.get_parsed_restql_query_from_req(request)
+        return None  # There is no query so we return None as a parsed query
 
     @property
     def fields(self):
-        if self._use_restql_fields:
-            # Use restql fields
+        dynamic_fields_disabled = self.restql_kwargs["disable_dynamic_fields"]
+        if self._ready_to_use_dynamic_fields and not dynamic_fields_disabled:
+            # Return restql fields
             return self.restql_fields
         return self.allowed_fields
 
@@ -492,19 +487,17 @@ class EagerLoadingMixin(RequestQueryParserMixin):
             if isinstance(item, str):
                 keys[item] = True
             elif isinstance(item, dict):
-                for key, nested_items in item.items():
-                    key_base = key
-                    nested_keys = cls.get_dict_parsed_restql_query(nested_items)
-                    keys[key_base] = nested_keys
+                nested_field = cls.get_nested_field_from_dict(item)
+                nested_keys = cls.get_dict_parsed_restql_query(nested_field.fields)
+                keys[nested_field.name] = nested_keys
 
         for item in exclude:
             if isinstance(item, str):
                 keys[item] = False
             elif isinstance(item, dict):
-                for key, nested_items in item.items():
-                    key_base = key
-                    nested_keys = cls.get_dict_parsed_restql_query(nested_items)
-                    keys[key_base] = nested_keys
+                nested_field = cls.get_nested_field_from_dict(item)
+                nested_keys = cls.get_dict_parsed_restql_query(nested_field.fields)
+                keys[nested_field.name] = nested_keys
         return keys
 
     @staticmethod
