@@ -2,6 +2,8 @@ import re
 
 from pypeg2 import List, contiguous, csl, name, optional, parse
 
+from .exceptions import QueryFormatError
+
 
 class Alias(List):
     grammar = name(), ':'
@@ -10,12 +12,6 @@ class Alias(List):
 class IncludedField(List):
     grammar = optional(Alias), name()
 
-    @property
-    def alias_or_name(self):
-        if self.alias:
-            return self.alias
-        return self.name
-
     @property
     def alias(self):
         if len(self) > 0:
@@ -34,7 +30,10 @@ class AllFields(str):
 class BaseArgument(List):
     @property
     def value(self):
-        return self[0]
+        try:
+            return self[0]
+        except IndexError:
+            return ""
 
 
 class ArgumentWithoutQuotes(BaseArgument):
@@ -42,11 +41,11 @@ class ArgumentWithoutQuotes(BaseArgument):
 
 
 class ArgumentWithSingleQuotes(BaseArgument):
-    grammar = name(), ':', "'", re.compile(r'[^\']+'), "'"
+    grammar = name(), ':', "'", optional(re.compile(r'[^\']+')), "'"
 
 
 class ArgumentWithDoubleQuotes(BaseArgument):
-    grammar = name(), ':', '"', re.compile(r'[^"]+'), '"'
+    grammar = name(), ':', '"', optional(re.compile(r'[^"]+')), '"'
 
 
 class Arguments(List):
@@ -84,12 +83,6 @@ class ParentField(List):
     def alias(self):
         return self[0].alias
 
-    @property
-    def alias_or_name(self):
-        if self.alias:
-            return self.alias
-        return self[0].name
-
     @property
     def block(self):
         return self[1]
@@ -153,7 +146,7 @@ class QueryParser(object):
                 # A field is a parent
                 fields["include"].append(field)
             elif isinstance(field, IncludedField):
-                fields["include"].append(str(field.alias_or_name))
+                fields["include"].append(str(field.name))
             elif isinstance(field, ExcludedField):
                 fields["exclude"].append(str(field.name))
             elif isinstance(field, AllFields):
@@ -162,6 +155,21 @@ class QueryParser(object):
 
         if fields["exclude"] and "*" not in fields["include"]:
             fields["include"].append("*")
+
+        field_names = set(fields["aliases"].values())
+        field_aliases = set(fields["aliases"].keys())
+        faulty_fields = field_names.intersection(field_aliases)
+        if faulty_fields:
+            # We check this here because if we let it pass during
+            # parsing it's going to raise inappropriate error message
+            # when checking fields availability(for the case of renamed parents)
+            msg = (
+                "You have either "
+                "used an existing field name as an alias to another field or "  # e.g {id, id: course{}}
+                "you have defined an alias with the same name as a field name."  # e.g {id: id}
+                "The list of fields which led to this error is %s."
+            ) % str(list(faulty_fields))
+            raise QueryFormatError(msg)
         return fields
 
     def _transform_field(self, field):
@@ -171,6 +179,6 @@ class QueryParser(object):
         return field
 
     def _transform_parent_field(self, parent_field):
-        parent_field_name = str(parent_field.alias_or_name)
+        parent_field_name = str(parent_field.name)
         parent_field_value = self._transform_block(parent_field.block)
         return {parent_field_name: parent_field_value}
